var canvas = document.getElementById("cvGame");
var ctx = canvas.getContext("2d");
var runway;

var Sprite = function(imgSrc) {
    this.x = 0;
    this.y = 0;
    this.moveX = 0;
    this.moveY = 0;
    this.img = new Image();
    this.img.src = imgSrc;
    this.continuous = false;
    this.visible = false;
};
Sprite.prototype.animate = function() {
    if(this.visible){
        ctx.drawImage(this.img, this.x, this.y);
    }
    if(game.state == game.states.running || game.state == game.states.over){
        this.x += this.moveX;
        this.y += this.moveY;
        // allow for continuous display of horizontal sprite
        if(this.continuous && this.x < (canvas.width - this.img.width)){
            this.x = 0;
        }
    }
};
var bird = new Sprite("images/flapping.png");
function setBird(){
    bird.x = Math.floor(canvas.width / 3);
    bird.y = canvas.height / 2;
    bird.visible = true;
    bird.frames = 4;
    bird.frame = -1;
    bird.frameWidth = 57;
    bird.rotation = 0;
    bird.angleUp = -15*Math.PI/180;
    bird.angleDip = 5*Math.PI/180;
    bird.maxDip = 70*Math.PI/180;
}
bird.animate = function(){
    if(game.state === game.states.start || 
        game.state === game.states.running){
        this.frame += 1;
        this.frame %= this.frames;
    } else {
        this.frame = 0; // stop animation at end
    }
    if(game.state === game.states.running || 
            game.state === game.states.over){
        this.y += this.moveY;
    }
    ctx.save();
    let xAdj = this.frameWidth / 2;
    let yAdj = this.img.height / 2;
    ctx.translate(this.x + xAdj, this.y + yAdj);
    ctx.rotate(this.rotation);
    let clipx = this.frame * this.frameWidth;
    ctx.drawImage(this.img, clipx, 0, this.frameWidth,
                    this.img.height, -xAdj, -yAdj,
                    this.frameWidth, this.img.height);
    ctx.restore();
};

var game = {
    states: Object.freeze({"start": 0, "running":1, "over": 2,"end" : 3}),
    speed: 40,
    move: -2,
    runwayHeight: 25,
    state: null,
    tmr: null,
    width: 320,
    height: 480,
    pipeHeight: 392,
    pipes: [],
    jump: -10,
    maxGravity: 10, // terminal velocity
    gravity: 1,
    score: 0,
    highScore: 0
};
function playerControl(e) {
    switch (game.state){
       case game.states.start:
           game.state = game.states.running;
           break;
       case game.states.running:
            bird.moveY = game.jump;
            break;
       case game.states.end:
           break;
   }
}

function addPipePair(x, gapTop, gapHeight) {
    let topPipe = new Sprite("images/pipet.png");
    topPipe.x = x;
    topPipe.y = gapTop - game.pipeHeight;
    topPipe.moveX = game.move;
    game.pipes.push(topPipe);
    let bottomPipe = new Sprite("images/pipeb.png");
    bottomPipe.x = x;
    bottomPipe.y = gapTop + gapHeight;
    bottomPipe.moveX = game.move;
    game.pipes.push(bottomPipe);
}
function createPipes(){
    addPipePair(500, 100, 145);
    addPipePair(800, 50, 145);
    addPipePair(1000, 250, 140);
    addPipePair(1200, 150, 125);
    addPipePair(1600, 100, 125);
    addPipePair(1800, 150, 120);
    addPipePair(2000, 200, 120);
    addPipePair(2200, 250, 120);
    addPipePair(2400, 30, 105);
    addPipePair(2700, 300, 105);
    addPipePair(3000, 100, 85);
    addPipePair(3300, 250, 85);
    addPipePair(3600, 50, 65);
    let finish = new Sprite("images/finish.png");
    finish.x = 3900;
    finish.moveX = game.move;
    game.pipes.push(finish); // as finish must move with pipes.
}
function drawPipes(){
    for(let p = 0; p < game.pipes.length; p++) {
        if(game.pipes[p].x > -(game.pipes[p].img.width) && game.pipes[p].x < canvas.width){
            game.pipes[p].visible = true;
        } else {
            game.pipes[p].visible = false;
        }
        game.pipes[p].animate();
    }
}
function intersect(rectA, rectB){
    return Math.max(rectA.x, rectB.x) < 
        Math.min(rectA.x + rectA.width, rectB.x + rectB.width) &&
        Math.max(rectA.y, rectB.y) < 
        Math.min(rectA.y + rectA.height, rectB.y + rectB.height);
}

var Rect = function(sprite){
    this.x = sprite.x;
    this.y = sprite.y;
    this.width = sprite.img.width;
    if(sprite.frames){
        this.width = sprite.frameWidth;
    }
    this.height = sprite.img.height;
};
function setRunway(){
    runway = new Sprite("images/runway.png");
    runway.x = 0;
    runway.y = canvas.height - game.runwayHeight;
    runway.moveX = game.move;
    runway.continuous = true;
    runway.visible = true;
}
function initialise() {
    canvas.width = game.width;
    canvas.height = game.height;
    setRunway();
    createPipes();
    setBird();
    window.addEventListener("keydown", playerControl);
    window.addEventListener("pointerdown", playerControl);
    game.state = game.states.start;
    game.tmr = setInterval(drawFrame, game.speed);
}
function playerControl(e) {
    switch (game.state){
       case game.states.start:
           game.state = game.states.running;
           break;
       case game.states.running:
            bird.moveY = game.jump;
           break;
       case game.states.end:
           break;
   }
}
function birdMove(){
    if(game.state === game.states.running || 
        game.state === game.states.over){
        if(bird.moveY < game.maxGravity){
            bird.moveY += game.gravity;
        }
    }
    if(bird.y > (canvas.height - (bird.img.height + 
                                game.runwayHeight))){
        game.state = game.states.end; // game ends here
        clearInterval(game.tmr); // stop the action
    }
    switch(true) {
        case bird.moveY <= -7:
            bird.rotation = bird.angleUp;
            break;
        case bird.moveY <=-4:
            bird.rotation = bird.angleUp + bird.angleDip;
            break;
        case bird.moveY <= 0:
            bird.rotation = 0;
            break;
        default:
            bird.rotation += (bird.rotation < bird.maxDip) ?
                                bird.angleDip : 0;
            break;
    }
}
function birdStrike(){
    let bRect = new Rect(bird);
    for(let p = 0; p < game.pipes.length; p++){
        if(game.pipes[p].visible){
            let pRect = new Rect(game.pipes[p]);
            if(intersect(bRect, pRect)){
                game.state = game.states.over;
                return;
            }
        }
    }
}
function drawFrame() {
    ctx.clearRect(0, 0, canvas.width, canvas.height);
    drawPipes();
    runway.animate();
    bird.animate();
    birdStrike();
    birdMove();
    if(game.state === game.states.start){
        showPlay("Click, touch or key to start");
    }
}
function showPlay(txt) {
    ctx.font= "24px Arial";
    ctx.fillStyle= "blue";
    ctx.textAlign="center";
    ctx.fillText(txt, canvas.width / 2, canvas.height / 2 + 60);  
}

initialise();